@@ -183,8 +183,10 @@ ebews_populate_uid	(EContact *contact, EEwsItem *item)
 	const EwsId *id;
 
 	id = e_ews_item_get_id (item);
-	e_contact_set (contact, E_CONTACT_UID, id->id);
-	e_contact_set (contact, E_CONTACT_REV, id->change_key);
+	if (id) {
+		e_contact_set (contact, E_CONTACT_UID, id->id);
+		e_contact_set (contact, E_CONTACT_REV, id->change_key);
+	}
 }
 
 static void
@@ -193,7 +195,8 @@ ebews_populate_full_name	(EContact *contact, EEwsItem *item)
 	const EwsCompleteName *cn;
 
 	cn = e_ews_item_get_complete_name (item);
-	e_contact_set (contact, E_CONTACT_FULL_NAME, cn->full_name);
+	if (cn)
+		e_contact_set (contact, E_CONTACT_FULL_NAME, cn->full_name);
 }
 
 static void
@@ -202,7 +205,8 @@ ebews_populate_nick_name	(EContact *contact, EEwsItem *item)
 	const EwsCompleteName *cn;
 
 	cn = e_ews_item_get_complete_name (item);
-	e_contact_set (contact, E_CONTACT_NICKNAME, cn->nick_name);
+	if (cn)
+		e_contact_set (contact, E_CONTACT_NICKNAME, cn->nick_name);
 }
 
 static void
@@ -357,6 +361,21 @@ ebews_set_full_name		(ESoapMessage *msg, EContact *contact)
 static void
 ebews_set_birth_date		(ESoapMessage *message, EContact *contact)
 {
+	EContactDate *date;
+	gchar *birthday;
+
+	date = e_contact_get(contact, E_CONTACT_BIRTH_DATE);
+
+	if (!date)
+		return;
+
+	birthday = g_strdup_printf("%04d-%02d-%02dT00:00:00",
+			      date->year, date->month, date->day);
+
+	e_ews_message_write_string_parameter(message, "Birthday", NULL, birthday);
+
+	g_free (birthday);
+
 	
 }
 
@@ -517,7 +536,20 @@ ebews_set_full_name_changes	(ESoapMessage *message, EContact *new, EContact *old
 static void
 ebews_set_birth_date_changes	(ESoapMessage *message, EContact *new, EContact *old)
 {
+	EContactDate *new_date, *old_date;
+	gchar *birthday;
 
+	new_date = e_contact_get(new, E_CONTACT_BIRTH_DATE);
+	old_date = e_contact_get(old, E_CONTACT_BIRTH_DATE);
+
+	if (e_contact_date_equal(new_date, old_date))
+		return;
+
+	birthday = g_strdup_printf("%04d-%02d-%02dT00:00:00",
+			      new_date->year, new_date->month, new_date->day);
+
+	convert_contact_property_to_updatexml(message, "Birthday", birthday, "contacts", NULL, NULL);
+	g_free (birthday);
 }
 
 static void
@@ -894,6 +926,7 @@ e_book_backend_ews_remove_contacts	(EBookBackend *backend,
 	EBookBackendEws *ebews;
 	EwsRemoveContact *remove_contact;
 	EBookBackendEwsPrivate *priv;
+	GSList *l, *copy = NULL;
  
 	ebews = E_BOOK_BACKEND_EWS (backend);
  
@@ -919,11 +952,15 @@ e_book_backend_ews_remove_contacts	(EBookBackend *backend,
 		return;
 	}
 
+	for (l = (GSList *) id_list; l != NULL; l = g_slist_next (id_list))
+		copy = g_slist_prepend (copy, g_strdup ((gchar *)l->data));
+	copy = g_slist_reverse (copy);
+
 	remove_contact = g_new0(EwsRemoveContact, 1);
 	remove_contact->ebews = g_object_ref(ebews);
 	remove_contact->book = g_object_ref(book);
 	remove_contact->opid = opid;
-	remove_contact->sl_ids = (GSList *) id_list;
+	remove_contact->sl_ids = copy;
 
 	e_ews_connection_delete_items_start (priv->cnc, EWS_PRIORITY_MEDIUM, (GSList *) id_list,
 					     EWS_HARD_DELETE, 0 , FALSE,
@@ -1021,16 +1058,14 @@ convert_contact_to_updatexml (ESoapMessage *msg, gpointer user_data)
 		if (element_type == ELEMENT_TYPE_SIMPLE)  {
 			value =  e_contact_get (new_contact, mappings[i].field_id);
 			old_value =  e_contact_get (old_contact, mappings[i].field_id);
+			if ((value && !old_value) || (!value && old_value) ||(value && old_value && g_ascii_strcasecmp(value, old_value)))
+				convert_contact_property_to_updatexml(msg, mappings[i].element_name, value, "contacts", NULL, NULL);
 			if (value)
-			{
-				if(( *value && !old_value) || g_ascii_strcasecmp (value, old_value))
-					convert_contact_property_to_updatexml(msg, mappings[i].element_name, value, "contacts", NULL, NULL);
 				g_free(value);
-			}
-			if(old_value)
+			if (old_value)
 				g_free(old_value);
 		} else if (element_type == ELEMENT_TYPE_COMPLEX) {
-			if(mappings [i].field_id == E_CONTACT_UID)
+			if (mappings [i].field_id == E_CONTACT_UID)
 				continue;
 			mappings[i].set_changes (msg, new_contact, old_contact);
 		}
@@ -2031,7 +2066,7 @@ e_book_backend_ews_start_book_view (EBookBackend  *backend,
 	gboolean is_autocompletion = FALSE;
 	gchar *auto_comp_str = NULL;
 	GCancellable *cancellable;
-	GSList *ids = NULL, *mailboxes = NULL, *l;
+	GSList *ids = NULL, *mailboxes = NULL, *l, *contacts = NULL, *c;
 	EwsFolderId *fid;
 	GError *error = NULL;
 	gboolean includes_last_item;
@@ -2045,7 +2080,7 @@ e_book_backend_ews_start_book_view (EBookBackend  *backend,
 	e_data_book_view_notify_progress (book_view, -1, _("Searching..."));
 
 	if (!priv->is_online) {
-		if (e_book_backend_sqlitedb_get_is_populated (priv->ebsdb, priv->folder_id, NULL)) {
+		if (priv->ebsdb && e_book_backend_sqlitedb_get_is_populated (priv->ebsdb, priv->folder_id, NULL)) {
 			fetch_from_offline (ebews, book_view, query, error);
 			return;
 		}
@@ -2097,7 +2132,7 @@ e_book_backend_ews_start_book_view (EBookBackend  *backend,
 	   find_items rather than resolve_names to support all queries */
 	g_hash_table_insert (priv->ops, book_view, cancellable);
 	e_ews_connection_resolve_names	(priv->cnc, EWS_PRIORITY_MEDIUM, auto_comp_str,
-					 EWS_SEARCH_AD, NULL, FALSE, &mailboxes, NULL,
+					 EWS_SEARCH_AD, NULL, TRUE, &mailboxes, &contacts,
 					 &includes_last_item, cancellable, &error);
 	g_free (auto_comp_str);
 	g_hash_table_remove (priv->ops, book_view);
@@ -2109,26 +2144,51 @@ e_book_backend_ews_start_book_view (EBookBackend  *backend,
 		return;
 	}
 
-	for (l = mailboxes; l != NULL; l = g_slist_next (l)) {
+	for (l = mailboxes, c = contacts; l != NULL; l = g_slist_next (l), c = c ? g_slist_next (c) : NULL) {
 		EwsMailbox *mb = l->data;
+		EwsResolveContact *rc = c ? c->data : NULL;
 		EContact *contact;
 
 		contact = e_contact_new ();
 
 		/* We do not get an id from the server, so just using email_id as uid for now */
 		e_contact_set (contact, E_CONTACT_UID, mb->email);
-		e_contact_set (contact, E_CONTACT_FULL_NAME, mb->name);
-		e_contact_set (contact, E_CONTACT_EMAIL_1, mb->email);
+
+		if (rc && rc->display_name && *rc->display_name)
+			e_contact_set (contact, E_CONTACT_FULL_NAME, rc->display_name);
+		else
+			e_contact_set (contact, E_CONTACT_FULL_NAME, mb->name);
+
+		if (rc && g_hash_table_size (rc->email_addresses) > 0) {
+			GList *emails = g_hash_table_get_values (rc->email_addresses), *iter;
+			GList *use_emails = NULL;
+
+			for (iter = emails; iter; iter = iter->next) {
+				if (iter->data && g_str_has_prefix (iter->data, "SMTP:"))
+					use_emails = g_list_prepend (use_emails, ((gchar *) iter->data) + 5);
+			}
+
+			if (!use_emails)
+				use_emails = g_list_prepend (use_emails, mb->email);
+
+			e_contact_set (contact, E_CONTACT_EMAIL, use_emails);
+
+			g_list_free (use_emails);
+			g_list_free (emails);
+		} else
+			e_contact_set (contact, E_CONTACT_EMAIL_1, mb->email);
 
 		e_data_book_view_notify_update (book_view, contact);
 
 		g_free (mb->email);
 		g_free (mb->name);
 		g_free (mb);
+		e_ews_free_resolve_contact (rc);
 		g_object_unref (contact);
 	}
 
 	g_slist_free (mailboxes);
+	g_slist_free (contacts);
 	e_data_book_view_notify_complete (book_view, error);
 	e_data_book_view_unref (book_view);
 }
@@ -2206,9 +2266,10 @@ e_book_backend_ews_load_source 	(EBookBackend           *backend,
 		}
 	}
 	
-	e_book_backend_notify_opened (backend, NULL);
-	if (priv->is_online)
+	if (priv->is_online) {
 		e_backend_set_online (E_BACKEND (backend), TRUE);
+		e_book_backend_notify_auth_required(backend, TRUE, NULL);
+	}
 }
 
 static void
@@ -2228,8 +2289,12 @@ e_book_backend_ews_authenticate_user (EBookBackend *backend,
 {
 	EBookBackendEws *ebgw;
 	EBookBackendEwsPrivate *priv;
+	EEwsConnection *cnc = NULL;
 	ESource *esource;
 	GError *error = NULL;
+	GSList *folders = NULL, *ids = NULL;
+	EwsFolderId *fid = NULL;
+
 	const gchar *host_url;
 	const gchar *read_only;
 
@@ -2250,7 +2315,7 @@ e_book_backend_ews_authenticate_user (EBookBackend *backend,
 	host_url = e_source_get_property (esource, "hosturl");
 	read_only = e_source_get_property (esource, "read_only");
 
-	priv->cnc = e_ews_connection_new (host_url, e_credentials_peek (credentials, E_CREDENTIALS_KEY_USERNAME), 
+	cnc = e_ews_connection_new (host_url, e_credentials_peek (credentials, E_CREDENTIALS_KEY_USERNAME), 
 					  e_credentials_peek (credentials, E_CREDENTIALS_KEY_PASSWORD),
 					  NULL, NULL, &error);
 
@@ -2259,11 +2324,28 @@ e_book_backend_ews_authenticate_user (EBookBackend *backend,
 	} else 
 		priv->is_writable = TRUE;
 
+	/* a dummy request to make sure we have a authenticated connection */
+	fid = g_new0 (EwsFolderId, 1);
+	fid->id = g_strdup ("contacts");
+	fid->is_distinguished_id = TRUE;
+	ids = g_slist_append (ids, fid);
+	e_ews_connection_get_folder (cnc, EWS_PRIORITY_MEDIUM, "Default", NULL, ids, &folders, NULL, &error);
+
+	e_ews_folder_free_fid (fid);
+	g_slist_free (ids);
+	ids = NULL;
+
+	if (error) {
+		e_book_backend_notify_auth_required (backend, TRUE, priv->credentials);
+		e_book_backend_notify_opened (backend, error);
+		g_object_unref (cnc);
+		return;
+	}
+
+	priv->cnc = cnc;
 	priv->username = e_source_get_duped_property (esource, "username");
 	priv->password = g_strdup (e_credentials_peek (credentials, E_CREDENTIALS_KEY_PASSWORD));
 
-	/* FIXME: Do some dummy request to ensure that the password is actually
-	   correct; don't just blindly return success */
 	e_book_backend_notify_opened (backend, EDB_ERROR (SUCCESS));
 	e_book_backend_notify_readonly (backend, !priv->is_writable);
 }	
